# ----------------- OBLIGATORIO 1 ----------------
# --------------- Script de ejemplo --------------
# Observaciones:
# --> Se simula una red WLAN (IEEE 802.11b) entre 1 AP y 3 nodos
# --> Tráfico saturado uplink (de los nodos al AP)
#
# Modificar lo necesario para la realización de los dos problemas del obligatorio.
#-------------------------------------------------

########################################
# Parámetros de entrada
########################################

if {$argc == 0} {
    set opt(nn) 3 
    set opt(xdist) 100
    set opt(run) 1

} elseif {$argc != 3 } {
    puts " invalid argc ($argc)"
    puts " usage: $argv0 numnodes xdist replicationnumber"
    exit
} else {

    set opt(nn)      [lindex $argv 0]       
    set opt(xdist)   [lindex $argv 1]
    set opt(run)     [lindex $argv 2]
}


########################################
# Otras variables
########################################

# Duración total
set opt(duration)     20

# Comienzo de la ejecución
set opt(startmin)     1
set opt(startmax)     1.20

# Trazas
set opt(resultdir) "/tmp"
set opt(tracedir) "/tmp"
set machine_name [exec uname -n]
set opt(fstr)        ${argv0}_${opt(nn)}_.${machine_name}
set opt(resultfname) "${opt(resultdir)}/stats_${opt(fstr)}.log"
set opt(tracefile)   "${opt(tracedir)}/${argv0}.tr"

########################################
# Librerías
########################################

remove-all-packet-headers

set system_type [exec uname -s]
# Modificar la ruta para encontrar correctamente la libreria
if {[string match "CYGWIN*" "$system_type"] == 1} {
    load ../src/.libs/cygdei80211mr-0.dll
} else {
     #load /usr/ns-allinone-2.34/lib/libdei80211mr.so
      load /home/crattaro/ns-allinone-2.35/lib/libdei80211mr.so
} 

########################################
# Instancia del simulador
########################################

set ns [new Simulator]


########################################
# Generadores de números aleatorios
########################################

global defaultRNG
set startrng [new RNG]

set rvstart [new RandomVariable/Uniform]
$rvstart set min_ $opt(startmin)
$rvstart set max_ $opt(startmax)
$rvstart use-rng $startrng

# Semilla para cada corrida de la simulación
for {set j 1} {$j < $opt(run)} {incr j} {
    $defaultRNG next-substream
    $startrng next-substream
}


########################################
# Algunos procedimientos
########################################

proc finish {} {
    global ns tf opt 
    puts "done!"
    $ns flush-trace
    close $tf
    print_stats
    $ns halt 
    puts " "
    puts "Tracefile     : $opt(tracefile)"
    puts "Results file  : $opt(resultfname)"    
}



proc print_stats  {} {
    global fh_cbr opt me_mac me_phy peerstats 

    set resultsFilePtr [open $opt(resultfname) w]

    for {set id 1} {$id <= $opt(nn) } {incr id}  {
	set snr [$me_mac($id) getAPSnr ]
	set peerstatsstr [$peerstats getPeerStats $id 0]
	set mestatsstr   [$me_mac($id) getMacCounters]
	set logstr "$opt(run) $opt(nn) $opt(duration) $id $snr  $mestatsstr $peerstatsstr "
 	puts $resultsFilePtr $logstr

    }

    close $resultsFilePtr

}


Node/MobileNode instproc getIfq { param0} {
    $self instvar ifq_    
    return $ifq_($param0) 
}

Node/MobileNode instproc getPhy { param0} {
    $self instvar netif_    
    return $netif_($param0) 
}



########################################
# Configuración
########################################

PeerStatsDB set VerboseCounters_ 1
PeerStatsDB/Static set debug_ 0

Mac/802_11/Multirate set VerboseCounters_ 1


Mac/802_11 set RTSThreshold_ 50000
Mac/802_11 set ShortRetryLimit_ 8
Mac/802_11 set LongRetryLimit_  5
Mac/802_11/Multirate set useShortPreamble_ true
Mac/802_11/Multirate set gSyncInterval_ 0.000005
Mac/802_11/Multirate set bSyncInterval_ 0.00001
Mac/802_11 set CWMin_         32
Mac/802_11 set CWMax_         1024


Phy/WirelessPhy set Pt_ 0.01
Phy/WirelessPhy set freq_ 2437e6
Phy/WirelessPhy set L_ 1.0


Queue/DropTail/PriQueue set Prefer_Routing_Protocols    1
Queue/DropTail/PriQueue set size_ 10

set noisePower 7e-11
Phy/WirelessPhy set CSTresh_ [expr $noisePower * 1.1]

Channel/WirelessChannel/PowerAware set distInterference_ 200


set tf [open $opt(tracefile) w]
$ns trace-all $tf

set channel   [new Channel/WirelessChannel/PowerAware]

set topo      [new Topography]
$topo load_flatgrid [expr $opt(xdist) + 1] [expr $opt(xdist) + 1]

set god       [create-god [expr $opt(nn) + 1]]

set per       [new PER]
$per set noise_ $noisePower
$per loadPERTable80211bIntersilHFA3861B

set peerstats [new PeerStatsDB/Static]
$peerstats numpeers [expr $opt(nn) + 1]




###############################
# Configuración de nodos
###############################


$ns node-config -adhocRouting DumbAgent \
            -llType  LL    \
            -macType Mac/802_11/Multirate \
            -ifqType Queue/DropTail/PriQueue \
            -ifqLen 50 \
            -antType Antenna/OmniAntenna \
            -propType Propagation/FreeSpace/PowerAware \
	    -phyType Phy/WirelessPhy/PowerAware \
	    -topoInstance $topo \
	    -agentTrace OFF \
            -routerTrace OFF \
	    -macTrace ON \
            -ifqTrace OFF \
	    -channel $channel



#######################################
# Access Point
#######################################


set bs_node        [$ns node]

set bs_mac [$bs_node getMac 0]
set bs_ifq [$bs_node getIfq 0]
set bs_phy [$bs_node getPhy 0]

$bs_mac nodes [expr $opt(nn) + 1]
$bs_mac  basicMode_ Mode1Mb
$bs_mac  dataMode_ Mode11Mb
$bs_mac  per $per
$bs_mac  PeerStatsDB $peerstats
set bs_pp  [new PowerProfile]
$bs_mac  powerProfile $bs_pp
$bs_phy   powerProfile $bs_pp

set bs_macaddr [$bs_mac id]
$bs_mac     bss_id $bs_macaddr

$bs_node random-motion 0
$bs_node set X_ 0
$bs_node set Y_ 0
$bs_node set Z_ 0



###############################
#  Nodos clientes
###############################

for {set id 1} {$id <= $opt(nn)} {incr id} {

    set me_node($id)   [$ns node]

    set me_mac($id) [$me_node($id) getMac 0]
    set me_ifq($id) [$me_node($id) getIfq 0]
    set me_phy($id) [$me_node($id) getPhy 0]
    
    $me_mac($id) nodes [expr $opt(nn) + 1]
    $me_mac($id)   basicMode_ Mode1Mb
    $me_mac($id)   dataMode_ Mode11Mb
    $me_mac($id)   per $per
    $me_mac($id)   PeerStatsDB $peerstats
    set me_pp($id)   [new PowerProfile]
    $me_mac($id)   powerProfile $me_pp($id) 
    $me_phy($id)    powerProfile $me_pp($id) 
    $me_mac($id)  bss_id $bs_macaddr

    $me_node($id) set X_ 5
    $me_node($id) set Y_ 0
    $me_node($id) set Z_ 0
    $me_node($id) random-motion 0


#####################################################
# Tráfico
#####################################################

set me_tran($id)   [new Agent/UDP]
$ns attach-agent   $me_node($id) $me_tran($id)
set me_cbr($id)    [new Application/Traffic/CBR]
$me_cbr($id)       attach-agent $me_tran($id)

set bs_tran($id)   [new Agent/Null]
$ns attach-agent $bs_node $bs_tran($id)

$ns connect $me_tran($id) $bs_tran($id) 

$me_cbr($id) set packetSize_      1000
$me_cbr($id) set rate_             [ expr 11000000 / ( $opt(nn) - 1) ]

# Registro los eventos
set cbr_start($id) [$rvstart value]
set cbr_stop($id) [expr $cbr_start($id) + $opt(duration)]

$ns at $cbr_start($id)  "$me_cbr($id) start"
$ns at $cbr_stop($id)   "$me_cbr($id) stop"


}



###############################
#  Comienzo la simulación
###############################

puts -nonewline "Simulating"


for {set i 1} {$i < 40} {incr i} {
    $ns at [expr $opt(startmax) + (($opt(duration) * $i)/ 40.0) ] "puts -nonewline . ; flush stdout"
}
$ns at [expr $opt(startmax) + $opt(duration) + 5] "finish"
$ns run
